Producer = LibStub( 'AceAddon-3.0'):NewAddon( 'Producer', 'AceEvent-3.0', 'AceConsole-3.0', 'AceHook-3.0', 'LibDebugLog-1.0')
if not Producer then return end -- already loaded and no upgrade necessary

local L = LibStub( 'AceLocale-3.0'):GetLocale( 'Producer')
local DEFAULTS = { 
	profile = {
		sorttype = L.Alpha,
		money = true,
		canmake = true,
		oranges = false,
		yellows = false,
		icons = true,
		smooth = false,
		enabled = true,
		groups = {},
	} 
}

--[[---------------------------------------------------------------------------------
  Class Setup
------------------------------------------------------------------------------------]]
Producer.name, Producer.localizedname = GetAddOnInfo( 'Producer')
Producer.version = format( L.Version, GetAddOnMetadata( 'Producer', 'Version') or 'unknown', RED_FONT_COLOR_CODE, 'Beta-WotLK', FONT_COLOR_CODE_CLOSE)

function Producer:OnInitialize()
	self:ToggleDebugLog( true)
	--register database events
	self.db = LibStub( 'AceDB-3.0'):New( 'ProducerDB', DEFAULTS, 'Default')
	self.db.RegisterCallback( self, 'OnNewProfile')
	self.db.RegisterCallback( self, 'OnProfileChanged')
	self.db.RegisterCallback( self, 'OnProfileCopied')
	self.db.RegisterCallback( self, 'OnProfileReset')
	self.db.RegisterCallback( self, 'OnProfileDeleted')
	self:InitConfig()
end

function Producer:OnEnable()
	self:RegisterEvent( 'ADDON_LOADED')
	self:RegisterEvent( 'TRADE_SKILL_SHOW')
	self:RegisterEvent( 'TRADE_SKILL_CLOSE')
	self:RegisterEvent( 'TRADE_SKILL_UPDATE')
	self:RegisterEvent( 'CRAFT_SHOW')
	self:RegisterEvent( 'CRAFT_CLOSE')
	self:RegisterEvent( 'CRAFT_UPDATE')
	self:RegisterEvent( 'SPELLS_CHANGED')
	self:RegisterEvent( 'UPDATE_TRADESKILL_RECAST')
	self:RegisterEvent( 'SKILL_LINES_CHANGED')
	self:RegisterEvent( 'BAG_UPDATE')
	if IsAddOnLoaded( 'Blizzard_TradeSkillUI') then
		self:Hock( 'TradeSkillFrame_Show', true)
	end
	if IsAddOnLoaded( 'Blizzard_CraftUI') then
		self:Hock( 'CraftFrame_Show', true)
	end
	self:Update()
	self:Debug( 'Producer enabled')
	self:Print( self.version, 'loaded')
end

function Producer:OnDisable()
	self:HideFrames()
end

function Producer:ADDON_LOADED( addon)
	if addon == 'Blizzard_TradeSkillUI' then
		self:Hock( 'TradeSkillFrame_Show', true)
	elseif addon == 'Blizzard_CraftUI' then
		self:Hock( 'CraftFrame_Show', true)
	end
end

function Producer:TradeSkillFrame_Show()
	self:Debug( 'Producer:TradeSkillFrame_Show()')
end

function Producer:CraftFrame_Show()
	local tradeskillName, currentLevel, maxLevel = GetTradeSkillLine()
	self:Debug( 'GetTradeSkillLine()', tradeskillName, currentLevel, maxLevel)
	local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo( tradeskillName)
	self:Debug( 'GetSpellInfo()', name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange)
--	if tradeskillName == BS['Enchanting'] then
--		return self:hooks['CraftFrame_Show']()
--	end
end

function Producer:TRADE_SKILL_SHOW( ...)
	self:Debug( 'Producer:TRADE_SKILL_SHOW()', ...)
end

function Producer:TRADE_SKILL_CLOSE( ...)
	self:Debug( 'Producer:TRADE_SKILL_CLOSE()', ...)
end

function Producer:TRADE_SKILL_UPDATE( ...)
	self:Debug( 'Producer:TRADE_SKILL_UPDATE()', ...)
end

function Producer:CRAFT_SHOW( ...)
	self:Debug( 'Producer:CRAFT_SHOW()', ...)
end

function Producer:CRAFT_CLOSE( ...)
	self:Debug( 'Producer:CRAFT_CLOSE()', ...)
end

function Producer:CRAFT_UPDATE( ...)
	self:Debug( 'Producer:CRAFT_UPDATE()', ...)
end

function Producer:SPELLS_CHANGED( ...)
	self:Debug( 'Producer:SPELLS_CHANGED()', ...)
	self:ScanActionButtons()
end

function Producer:UPDATE_TRADESKILL_RECAST( ...)
	self:Debug( 'Producer:UPDATE_TRADESKILL_RECAST()', ...)
end

function Producer:SKILL_LINES_CHANGED( ...)
	self:Debug( 'Producer:SKILL_LINES_CHANGED()', ...)
	self:ScanActionButtons()
end

function Producer:BAG_UPDATE( ...)
	self:Debug( 'Producer:BAG_UPDATE()', ...)
end

--[[---------------------------------------------------------------------------------
  Main Methods
------------------------------------------------------------------------------------]]
function Producer:Update()
	if self.db.profile.enabled then
	end
end

function Producer:ScanActionButtons()
end

function Producer:PrintHelp()
	local function PrintCmd( cmd, desc) self:Print( format( SUB_HELP_MSG, cmd, desc)) end
	self:Print( 'Commands (/pr, /prod, /producer)')
	PrintCmd( 'enable', L.EnabledDesc)
	PrintCmd( 'version', L.VersionDesc)
end

function Producer:PrintVersion() 
	self:Print( self.version) 
end

--[[---------------------------------------------------------------------------------
  Set/Get Methods
------------------------------------------------------------------------------------]]
function Producer:IsEnabled()
	return self.db.profile.enabled
end

function Producer:SetEnabled( value)
	self.db.profile.enabled = value
end

function Producer:ToggleEnabled() 
	self:SetEnabled( not self:IsEnabled()) 
end

--[[---------------------------------------------------------------------------------
Profile Functions
------------------------------------------------------------------------------------]]
function Producer:ListProfiles()
	self:Print( L.AvailableProfiles)
	local current = self.db:GetCurrentProfile()
	for _,name in ipairs(self.db:GetProfiles()) do
		self:Print( format( (name == current) and ' - |cFFFFFF00%s|r' or ' - %s', name))
	end
end

function Producer:OnNewProfile( event, db, name)
	self:Update()
	self:Print( format( L.ProfileCreated, name))
end

function Producer:OnProfileChanged( event, db, name)
	self:Update()
	self:Print( format( L.ProfileLoaded, name))
end

function Producer:OnProfileCopied( event, db, name)
	self:Update()
	self:Print( format( L.ProfileCopied, name))
end

function Producer:OnProfileReset( event, db)
	self:Update()
	self:Print( format( L.ProfileReset, db:GetCurrentProfile()))
end

function Producer:OnProfileDeleted( event, db, name)
	self:Print( format( L.ProfileDeleted, name))
end

--[[---------------------------------------------------------------------------------
  create a widget class
------------------------------------------------------------------------------------]]
function Producer:CreateClass(type, parentClass)
	local class = CreateFrame( type)
	class.mt = { __index = class }
	if parentClass then
		class = setmetatable( class, { __index = parentClass })
		class.super = parentClass
	end

	function class:Bind( o)
		return setmetatable( o, self.mt)
	end

	return class
end

